/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.bpmn.views {
  import flash.display.DisplayObject;
  import flash.events.Event;

  import mx.binding.utils.BindingUtils;
  import mx.binding.utils.ChangeWatcher;
  import mx.containers.Canvas;
  import mx.controls.Image;
  import mx.controls.Label;
  import mx.events.CollectionEvent;

  import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
  import org.ow2.orchestra.designer.bpmn.model.EventModel;
  import org.ow2.orchestra.designer.bpmn.model.EventType;

  public class EventView extends AbstractElementView {
    [Bindable]
    private var _eventModel:EventModel;

    /**
     * The source for "event" background image data binding.
     */
    [Bindable]
    [Embed(source="/../assets/startEvent.svg")]
    public var startEventImage:Class;
    [Bindable]
    [Embed(source="/../assets/intermediateEvent.svg")]
    public var intermediateEventImage:Class;
    [Bindable]
    [Embed(source="/../assets/endEvent.svg")]
    public var endEventImage:Class;
    /**
     * The source for "event" image data binding.
     */
    [Bindable]
    [Embed(source="/../assets/receiveMessageEvent.svg")]
    public var receiveMessageEventImage:Class;
    [Bindable]
    [Embed(source="/../assets/sendMessageEvent.svg")]
    public var sendMessageEventImage:Class;
    [Bindable]
    [Embed(source="/../assets/timerEvent.svg")]
    public var timerEventImage:Class;

    private var imageEvent:Image = new Image();
    private var _imageBackgroundEvent:Image = new Image();


    function EventView(eventModel:EventModel) {
      _eventModel = eventModel;
      this.setStyle("horizontalAlign", "center");
      imageEvent.scaleContent = false;
      imageEvent.automationName = "EventImage" + _eventModel.id;
      _imageBackgroundEvent.scaleContent = false;
      _imageBackgroundEvent.automationName = "EventBackgroundImage" + _eventModel.id;

      watchers.addItem(BindingUtils.bindProperty(this, "x", _eventModel, "x"));
      watchers.addItem(BindingUtils.bindProperty(this, "y", _eventModel, "y"));

      var label:Label = new Label();
      watchers.addItem(BindingUtils.bindProperty(label, "text", _eventModel, "name"));
      _eventModel.incomingSequenceFlows.addEventListener(CollectionEvent.COLLECTION_CHANGE, updateImage);
      _eventModel.outgoingSequenceFlows.addEventListener(CollectionEvent.COLLECTION_CHANGE, updateImage);
      watchers.addItem(ChangeWatcher.watch(_eventModel, "eventType", updateImage));

      var imageCanvas:Canvas = new Canvas();
      imageCanvas.addChild(_imageBackgroundEvent);
      imageCanvas.addChild(imageEvent);
      this.addChild(imageCanvas);
      this.addChild(label);
      updateImage();
    }


    override public function get model():AbstractElement {
      return _eventModel;
    }

    override public function copy():AbstractElementView {
      return new EventView(_eventModel);
    }

    override public function get borderImage():DisplayObject {
      return _imageBackgroundEvent;
    }

    override public function get subtypeImage():DisplayObject {
        return imageEvent;
    }

    override public function dispose():void {
      super.dispose();
      _eventModel.incomingSequenceFlows.removeEventListener(CollectionEvent.COLLECTION_CHANGE, updateImage);
      _eventModel.outgoingSequenceFlows.removeEventListener(CollectionEvent.COLLECTION_CHANGE, updateImage);
    }

    protected function updateImage(event:Event = null):void {
      // update front image (event type)
      if (_eventModel.eventType == EventType.RECEIVE_MESSAGE) {
        imageEvent.source = receiveMessageEventImage;
      } else if (_eventModel.eventType == EventType.SEND_MESSAGE) {
        imageEvent.source = sendMessageEventImage;
      } else if (_eventModel.eventType == EventType.TIMER) {
        imageEvent.source = timerEventImage;
      } else if (_eventModel.eventType == EventType.NONE) {
        imageEvent.source = null;
      }
      // Update background image (start/end/intermediate event)
      if (_eventModel.incomingSequenceFlows.length == 0) {
        _imageBackgroundEvent.source = startEventImage;
      } else if (_eventModel.outgoingSequenceFlows.length == 0) {
        _imageBackgroundEvent.source = endEventImage;
      } else {
        _imageBackgroundEvent.source = intermediateEventImage;
      }
    }

    public function get imageBackgroundEvent():Image {
      return _imageBackgroundEvent;
    }
  }
}
