/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.bpmn.views.utils {
  import flash.display.DisplayObject;
  import flash.display.DisplayObjectContainer;
  import flash.geom.Point;

  public class GraphicUtils {
    public function GraphicUtils() {
    }

    public static function findIntersection(image:DisplayObject, inPoint:Point, extPoint:Point, container:DisplayObjectContainer):Point {
      var x:Number;
      var y:Number;
      var p:Point;
      var incr:Number = 1 / Math.max(Math.abs(inPoint.x - extPoint.x), Math.abs(inPoint.y - extPoint.y));
      for (var i:Number = 0; i <= 1; i += incr) {
        p = Point.interpolate(inPoint, extPoint, i);
        if (image.hitTestPoint(p.x, p.y, true)) {
          // p is the first point of the line and of the image, it is the intersection point
          return p;
        }
      }

      return inPoint;
    }

    /**
     * Return the distance from a point to a segment
     *
     * @param ps the start of the segment
     * @param pe the end of the segment
     * @param p the given point
     * @return the distance from the given point to the segment
     */
    public static function distanceToSegment(ps:Point, pe:Point, p:Point):Number {

      if (ps.x == pe.x && ps.y == pe.y) {
        return Point.distance(ps, p);
      }

      var sx:Number = pe.x - ps.x;
      var sy:Number = pe.y - ps.y;

      var ux:Number = p.x - ps.x;
      var uy:Number = p.y - ps.y;

      var dp:Number = sx * ux + sy * uy;
      if (dp < 0) {
        return Point.distance(ps, p);
      }

      var sn2:Number = sx * sx + sy * sy;
      if (dp > sn2) {
        return Point.distance(pe, p);
      }

      var ah2:Number = dp * dp / sn2;
      var un2:Number = ux * ux + uy * uy;
      return Math.sqrt(un2 - ah2);
    }
  }
}
